//
// Created by philipp on 29.11.17.
//

#ifndef PROTOCOL_PLATOONINGPROTOCOL_H
#define PROTOCOL_PLATOONINGPROTOCOL_H

#include <cstdint>
#include <string>
#include <iostream>
#include <mutex>

namespace platoonProtocol
{
using VehicleId = std::uint32_t;
using MessageType = std::uint32_t;
using PlatoonId = std::uint32_t;
using PlatoonSpeed = float;
using InnerPlatoonDistance = float;

struct PlatoonConfig
{
    static constexpr float EPSILON{0.00001f};

    PlatoonConfig() : platoonSpeed(0.0f), innerPlatoonDistance(0.0f)
    {}

    PlatoonConfig(PlatoonSpeed platoonSpeed, InnerPlatoonDistance innerPlatoonDistance)
        : platoonSpeed(platoonSpeed), innerPlatoonDistance(innerPlatoonDistance)
    {}

    PlatoonSpeed platoonSpeed;
    InnerPlatoonDistance innerPlatoonDistance;
};

bool operator==(const PlatoonConfig & lhs, const PlatoonConfig & rhs) noexcept;

bool operator!=(const PlatoonConfig & lhs, const PlatoonConfig & rhs) noexcept;

class Log
{
public:
    virtual ~Log() = default;

    using Manipulator = Log & (*)(Log &);

    virtual Log & operator<<(const std::string & value)
    {
        std::cout << value;
        return *this;
    }

    virtual Log & operator<<(int value)
    {
        std::cout << value;
        return *this;
    }

    virtual Log & operator<<(float value)
    {
        std::cout << value;
        return *this;
    }

    friend Log & operator<<(Log & log, Manipulator manipulator)
    {
        return manipulator(log);
    }

    virtual Log & doFlush()
    {
        std::cout.flush();
        return *this;
    }

    static Log & flush(Log & log)
    {
        return log.doFlush();
    }
};

class NoLog : public Log
{
    Log & operator<<(const std::string & value) override
    { return *this; }

    Log & operator<<(int value) override
    { return *this; }

    Log & operator<<(float value) override
    { return *this; }

    Log & doFlush() override
    { return *this; }
};

Log & log();

void setLog(Log & log);

}

#endif //PROTOCOL_PLATOONINGPROTOCOL_H
